# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: flights.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'flights.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rflights.proto\"\x1a\n\x07\x41irport\x12\x0f\n\x07\x61irport\x18\x02 \x01(\t\"\x8e\x01\n\nFlightData\x12\x0c\n\x04\x64\x61te\x18\x02 \x01(\t\x12\x1d\n\x0b\x66rom_flight\x18\r \x01(\x0b\x32\x08.Airport\x12\x1b\n\tto_flight\x18\x0e \x01(\x0b\x32\x08.Airport\x12\x16\n\tmax_stops\x18\x05 \x01(\x05H\x00\x88\x01\x01\x12\x10\n\x08\x61irlines\x18\x06 \x03(\tB\x0c\n\n_max_stops\"\xa9\x01\n\x04Info\x12\x19\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x0b.FlightData\x12\x13\n\x04seat\x18\t \x01(\x0e\x32\x05.Seat\x12\x1e\n\npassengers\x18\x08 \x03(\x0e\x32\n.Passenger\x12\x13\n\x04trip\x18\x13 \x01(\x0e\x32\x05.Trip\x12\"\n\x15\x65xclude_basic_economy\x18\x19 \x01(\x08H\x00\x88\x01\x01\x42\x18\n\x16_exclude_basic_economy\"(\n\x05Price\x12\r\n\x05price\x18\x01 \x01(\x05\x12\x10\n\x08\x63urrency\x18\x03 \x01(\t\":\n\x10ItinerarySummary\x12\x0f\n\x07\x66lights\x18\x02 \x01(\t\x12\x15\n\x05price\x18\x03 \x01(\x0b\x32\x06.Price*S\n\x04Seat\x12\x10\n\x0cUNKNOWN_SEAT\x10\x00\x12\x0b\n\x07\x45\x43ONOMY\x10\x01\x12\x13\n\x0fPREMIUM_ECONOMY\x10\x02\x12\x0c\n\x08\x42USINESS\x10\x03\x12\t\n\x05\x46IRST\x10\x04*E\n\x04Trip\x12\x10\n\x0cUNKNOWN_TRIP\x10\x00\x12\x0e\n\nROUND_TRIP\x10\x01\x12\x0b\n\x07ONE_WAY\x10\x02\x12\x0e\n\nMULTI_CITY\x10\x03*_\n\tPassenger\x12\x15\n\x11UNKNOWN_PASSENGER\x10\x00\x12\t\n\x05\x41\x44ULT\x10\x01\x12\t\n\x05\x43HILD\x10\x02\x12\x12\n\x0eINFANT_IN_SEAT\x10\x03\x12\x11\n\rINFANT_ON_LAP\x10\x04\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'flights_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SEAT']._serialized_start=464
  _globals['_SEAT']._serialized_end=547
  _globals['_TRIP']._serialized_start=549
  _globals['_TRIP']._serialized_end=618
  _globals['_PASSENGER']._serialized_start=620
  _globals['_PASSENGER']._serialized_end=715
  _globals['_AIRPORT']._serialized_start=17
  _globals['_AIRPORT']._serialized_end=43
  _globals['_FLIGHTDATA']._serialized_start=46
  _globals['_FLIGHTDATA']._serialized_end=188
  _globals['_INFO']._serialized_start=191
  _globals['_INFO']._serialized_end=360
  _globals['_PRICE']._serialized_start=362
  _globals['_PRICE']._serialized_end=402
  _globals['_ITINERARYSUMMARY']._serialized_start=404
  _globals['_ITINERARYSUMMARY']._serialized_end=462
# @@protoc_insertion_point(module_scope)
